import {
  Button,
  Paper,
  TextField,
  Select,
  MenuItem,
  FormControl,
  InputLabel,
} from "@mui/material";
import { observer } from "mobx-react-lite";
import { ArrowLeft, Save } from "lucide-react";
import { Loader2 } from "lucide-react";
import { useNavigate } from "react-router-dom";
import { toast } from "react-toastify";
import { carrierStore, cityStore, mediaStore, languageStore } from "@shared";
import { useState, useEffect } from "react";
import { MediaViewer, ImageUploadCard, LanguageSwitcher } from "@widgets";
import {
  SelectMediaDialog,
  UploadMediaDialog,
  PreviewMediaDialog,
} from "@shared";

export const CarrierCreatePage = observer(() => {
  const navigate = useNavigate();
  const { language } = languageStore;
  const [fullName, setFullName] = useState("");
  const [shortName, setShortName] = useState("");
  const [cityId, setCityId] = useState<number | null>(null);
  const [slogan, setSlogan] = useState("");
  const [selectedMediaId, setSelectedMediaId] = useState<string | null>(null);
  const [isLoading, setIsLoading] = useState(false);
  const [isSelectMediaOpen, setIsSelectMediaOpen] = useState(false);
  const [isUploadMediaOpen, setIsUploadMediaOpen] = useState(false);
  const [isPreviewMediaOpen, setIsPreviewMediaOpen] = useState(false);
  const [mediaId, setMediaId] = useState("");
  const [activeMenuType, setActiveMenuType] = useState<
    "thumbnail" | "watermark_lu" | "watermark_rd" | null
  >(null);

  useEffect(() => {
    cityStore.getCities("ru");
    mediaStore.getMedia();
  }, []);

  const handleCreate = async () => {
    try {
      setIsLoading(true);
      await carrierStore.createCarrier(
        fullName,
        shortName,
        cityId!,
        slogan,
        selectedMediaId!
      );
      toast.success("Перевозчик успешно создан");
      navigate("/carrier");
    } catch (error) {
      toast.error("Ошибка при создании перевозчика");
    } finally {
      setIsLoading(false);
    }
  };

  const handleMediaSelect = (media: {
    id: string;
    filename: string;
    media_name?: string;
    media_type: number;
  }) => {
    setSelectedMediaId(media.id);
  };

  const selectedMedia = selectedMediaId
    ? mediaStore.media.find((m) => m.id === selectedMediaId)
    : null;

  return (
    <Paper className="w-full h-full p-3 flex flex-col gap-10">
      <LanguageSwitcher />
      <div className="flex items-center gap-4">
        <button
          className="flex items-center gap-2"
          onClick={() => navigate(-1)}
        >
          <ArrowLeft size={20} />
          Назад
        </button>
      </div>

      <div className="flex flex-col gap-10 w-full items-end">
        <div className="flex gap-10 items-center mb-5 max-w-[80%]">
          <h1 className="text-3xl break-words">Создание перевозчика</h1>
        </div>

        <FormControl fullWidth>
          <InputLabel>Город</InputLabel>
          <Select
            value={cityId || ""}
            label="Город"
            required
            onChange={(e) => setCityId(e.target.value as number)}
          >
            {cityStore.cities.ru.data.map((city) => (
              <MenuItem key={city.id} value={city.id}>
                {city.name}
              </MenuItem>
            ))}
          </Select>
        </FormControl>

        <TextField
          fullWidth
          label="Полное название"
          value={fullName}
          required
          onChange={(e) => setFullName(e.target.value)}
        />

        <TextField
          fullWidth
          label="Короткое название"
          value={shortName}
          required
          onChange={(e) => setShortName(e.target.value)}
        />

        <TextField
          fullWidth
          label="Слоган"
          value={slogan}
          onChange={(e) => setSlogan(e.target.value)}
        />

        <div className="w-full flex flex-col gap-4 max-w-[300px] mx-auto">
          <ImageUploadCard
            title="Логотип перевозчика"
            imageKey="thumbnail"
            imageUrl={selectedMedia?.id}
            onImageClick={() => {
              setIsPreviewMediaOpen(true);
              setMediaId(selectedMedia?.id ?? "");
            }}
            onDeleteImageClick={() => {
              setSelectedMediaId(null);
              setActiveMenuType(null);
            }}
            onSelectFileClick={() => {
              setActiveMenuType("thumbnail");
              setIsSelectMediaOpen(true);
            }}
            setUploadMediaOpen={() => {
              setIsUploadMediaOpen(true);
              setActiveMenuType("thumbnail");
            }}
          />
        </div>

        <Button
          variant="contained"
          className="w-min flex gap-2 items-center"
          startIcon={<Save size={20} />}
          onClick={handleCreate}
          disabled={
            isLoading || !fullName || !shortName || !cityId || !selectedMediaId
          }
        >
          {isLoading ? (
            <Loader2 size={20} className="animate-spin" />
          ) : (
            "Создать"
          )}
        </Button>
      </div>

      <SelectMediaDialog
        open={isSelectMediaOpen}
        onClose={() => setIsSelectMediaOpen(false)}
        onSelectMedia={handleMediaSelect}
        mediaType={3}
      />

      <UploadMediaDialog
        open={isUploadMediaOpen}
        onClose={() => setIsUploadMediaOpen(false)}
        afterUpload={handleMediaSelect}
        hardcodeType={activeMenuType}
      />

      <PreviewMediaDialog
        open={isPreviewMediaOpen}
        onClose={() => setIsPreviewMediaOpen(false)}
        mediaId={mediaId}
      />
    </Paper>
  );
});
